//------------------------------------------------------------------------------
// BigIni demo                                                     eh 2004-08-17
//------------------------------------------------------------------------------
unit uDemo;

interface

uses
  Messages, SysUtils, classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Windows, ShellAPI,
  BigIni, ExtCtrls;

const
  cComment  = ';comment';

type

  TFormMain = class(TForm)
    Memo1: TMemo;
    Memo2: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    FontDialog1: TFontDialog;
    Panel1: TPanel;
    CheckBoxDropCommentLines: TCheckBox;
    CheckBoxFilterOutInvalid: TCheckBox;
    CheckBoxDropWhiteSpace: TCheckBox;
    CheckBoxDropApostrophes: TCheckBox;
    ButtonDemo: TButton;
    Panel2: TPanel;
    ButtonFontTest: TButton;
    LabelFontDemo1: TLabel;
    LabelFontDemo2: TLabel;
    Panel3: TPanel;
    ButtonShowMore: TButton;
    LabelDateTime: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ButtonDemoClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ButtonFontTestClick(Sender: TObject);
    procedure ButtonShowMoreClick(Sender: TObject);
  private
    BigIniFile : TBiggerIniFile;
    FFileName  : string;
  public
    { none }
  end;

var
  FormMain: TFormMain;

implementation

{$R *.DFM}

procedure TFormMain.FormCreate(Sender: TObject);
begin
  FFileName  := ExtractFilePath(Application.ExeName)+'demo.ini';
  BigIniFile := TBiggerIniFile.Create(''); { no filename now }
  with BigIniFile do
  begin
    CheckBoxDropCommentLines.Checked := FlagDropCommentLines;
    CheckBoxFilterOutInvalid.Checked := FlagFilterOutInvalid;
    CheckBoxDropWhiteSpace.Checked   := FlagDropWhiteSpace;
    CheckBoxDropApostrophes.Checked  := FlagDropApostrophes;
  end;
end;

procedure TFormMain.ButtonDemoClick(Sender: TObject);
begin
  { save Lines to disk (again) to get test data: }
  Memo1.Lines.SaveToFile(FFileName);

  with BigIniFile do
  begin
    FlagDropCommentLines := CheckBoxDropCommentLines.Checked;
    FlagFilterOutInvalid := CheckBoxFilterOutInvalid.Checked;
    FlagDropWhiteSpace   := CheckBoxDropWhiteSpace.Checked;
    FlagDropApostrophes  := CheckBoxDropApostrophes.Checked;

    { load file: }
    FileName := FFileName;

    { now, load the result to the second memo: }
    Memo2.Lines.Clear;
    GetStrings(Memo2.Lines);

    FileName := '';
  end;
end;


procedure TFormMain.FormDestroy(Sender: TObject);
begin
  BigIniFile.free;
end;

procedure TFormMain.ButtonFontTestClick(Sender: TObject);
{
This demo shows the ReadFont/WriteFont features.
If you select any font, the first label (LabelFontDemo1) will get that font.
Then, we save it to INI.
Next, we read it from INI and assign it to LabelFontDemo2
}
begin
  { Let user chose any font: }
  FontDialog1.Font.Assign(LabelFontDemo1.Font);
  if FontDialog1.Execute then
  begin
    LabelFontDemo1.Font.Assign(FontDialog1.Font);

    with BigIniFile do
    begin

      { load file: }
      FileName := FFileName;

      { write first label font properties to INI: }
      WriteFont('FontSample','LabelFont',LabelFontDemo1.Font);
      { read font properties from INI to second label's font: }

      ReadFont('FontSample','LabelFont',LabelFontDemo2.Font);

      { now, show the result in the second memo field: }
      Memo2.Lines.Clear;
      GetStrings(Memo2.Lines);

      FileName := '';
    end;
  end;
end;

procedure TFormMain.ButtonShowMoreClick(Sender: TObject);
{
  tries to demonstrate some more features, possibly easier to show than to explain
}
var
  MyDskFile,
  MyIniFile             : TBiggerIniFile;
  tempStringList        : TStringList;
  someBinData           : String;
  sLen                  : Integer;
  BinBuff               : Pointer;
const
  BuffSize              = 2000;
begin
  MyIniFile := TBiggerIniFile.Create(ExtractFilePath(Application.ExeName)+'anotherDemo.ini');
  MyIniFile.Clear; // in case it existed before with "old" data
  tempStringList := TStringList.Create;

  // Demonstrate WriteSectionValues; writing comments
  tempStringList.Add('; This file demonstrates some features of Unit BigIni');
  tempStringList.Add('; (You may delete this file whenever wanted)');
  tempStringList.Add('; See BiniDemo.dpr source code to learn about this generated file.');
  tempStringList.Add('');
  tempStringList.Add('; Today''s special: WriteNumberedList() and ReadNumberedList()');
  MyIniFile.WriteSectionValues('programmer''s note',TempStringList);

  // Now demonstrate WriteNumberedList
  tempStringList.Clear;
  tempStringList.Add('Computing');
  tempStringList.Add('Music');
  tempStringList.Add('Shopping');
  // Standard: no prefix; index starting with 1
  MyIniFile.WriteString      ('Hobbies (1)',cComment,'Standard: no prefix; index starting with 1');
  MyIniFile.WriteNumberedList('Hobbies (1)',tempStringList);
  // all keys get the prefix 'Hobby_'
  MyIniFile.WriteString      ('Hobbies (2)',cComment,'all keys get the prefix ''Hobby_''');
  MyIniFile.WriteNumberedList('Hobbies (2)',tempStringList,'Hobby_');
  // all keys get the prefix 'Hobby_', index starting with 0
  MyIniFile.WriteString      ('Hobbies (3)',cComment,'all keys get the prefix ''Hobby_'', index starting with 0');
  MyIniFile.WriteNumberedList('Hobbies (3)',tempStringList,'Hobby_',0);
  // no prefix, index starting with 0
  MyIniFile.WriteString      ('Hobbies (4)',cComment,'no prefix, index starting with 0');
  MyIniFile.WriteNumberedList('Hobbies (4)',tempStringList,'',0);

  // all keys get the prefix 'Hobby_', the count-variables, too
  MyIniFile.WriteString      ('various',cComment,'all keys get the prefix ''Hobby_'', the count-variables, too');
  MyIniFile.WriteNumberedList('various',tempStringList,'Hobby_',1,true);
  tempStringList.Clear;
  tempStringList.Add('blue');
  tempStringList.Add('red');
  tempStringList.Add('pink');
  tempStringList.Add('cyan');
  // all keys get the prefix 'Colors_', the count-variables, too
  MyIniFile.WriteNumberedList('various',tempStringList,'Colors_',1,true);

  // prefixed lists with index 0 are typical for Delphi *.dsk files.
  // let's have a try:
  if fileExists(ChangeFileExt(Application.ExeName,'.dsk')) then
  begin
    // write a note:
    tempStringList.Clear;
    tempStringList.Add('; these data are from your current Delphi *.dsk file:');
    MyIniFile.WriteSectionValues('Modules',TempStringList);
    // you MUST clear the StringList used in ReadNumberedList; otherwise, it will be appended
    tempStringList.Clear;
    // read info from '*.dsk':
    MyDskFile := TBiggerIniFile.Create(ChangeFileExt(Application.ExeName,'.dsk'));
    MyDskFile.ReadNumberedList('Modules',tempStringList,'','Module',0);
    // write info to demo INI:
    MyIniFile.WriteNumberedList('Modules',tempStringList,'Module',0);
    MyDskFile.Free;
  end;

  // write binary data
  someBinData := FormatDateTime('ddd'#9'dd.mm.yyyy',now);
  MyIniFile.WriteBinaryData('time','now',someBinData,length(someBinData));
  // read binary data
  getmem(BinBuff,BuffSize);
  slen   := MyIniFile.ReadBinaryData('time','now',BinBuff,BuffSize);
  if (sLen > 0) then
  begin
    someBinData := Copy(String(BinBuff),1,sLen);
    LabelDateTime.Caption := someBinData;
  end;
  FreeMem(BinBuff,BuffSize);


  MyIniFile.FlushFile;
  // Now launch the editor
  MyIniFile.LaunchInEditor;

  tempStringList.Free;
  MyIniFile.Free;
end;

end.
